import sys
import os
import time
import math
import shutil
import random
import multiprocessing
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_utils import ensure_dir

def extractThresholds(input_fn):
    thrs = set([])

    if not os.path.exists(input_fn):
        return set([])
    input_fh = open(input_fn)
    lines = input_fh.readlines()
    for line in lines:
        tokens = line.split()
        if len(tokens) < 4: continue
        drug_name_id = tokens[0]
        mutkey_id = tokens[1]
        all_db = tokens[2]
        hc_db = tokens[3]
        gene_all_db = tokens[4]
        gene_gc_db = tokens[5]
        sc_val = float(tokens[6])
        thrs.add(sc_val)
    return thrs


def selectAssocs(input_fn, subset="all1", frac=0.1):
    positives = []
    negatives = []
    negatives_all = []

    pos_sel = set([])
    neg_sel = set([])

    if not os.path.exists(input_fn):
        return None, None
    input_fh = open(input_fn)
    lines = input_fh.readlines()
    k = len(lines)
    for i in range(k):
        tokens = lines[i].split()
        if len(tokens) < 4: continue
        drug_name_id = tokens[0]
        mutkey_id = tokens[1]
        all_db = tokens[2]
        hc_db = tokens[3]
        gene_all_db = tokens[4]
        gene_hc_db = tokens[5]
        sc_val = float(tokens[6])

        if subset.endswith("1"):
            if hc_db == "Y":  positives.append((hc_db, sc_val, drug_name_id, mutkey_id))
            else:  negatives.append((hc_db, sc_val, drug_name_id, mutkey_id))
            if hc_db == "N": 
                negatives_all.append((hc_db, sc_val, drug_name_id, mutkey_id))
        elif subset.endswith("0"):
            if all_db == "Y": positives.append((all_db, sc_val, drug_name_id, mutkey_id))
            else: negatives.append((all_db, sc_val, drug_name_id, mutkey_id))
            if all_db == "N": 
                negatives_all.append((hc_db, sc_val, drug_name_id, mutkey_id))
        elif subset.endswith("3"):
            if hc_db == "Y" and gene_hc_db == "Y": positives.append((hc_db, sc_val, drug_name_id, mutkey_id))
            elif gene_hc_db == "Y": negatives.append((hc_db, sc_val, drug_name_id, mutkey_id))
            if hc_db == "N": 
                negatives_all.append((hc_db, sc_val, drug_name_id, mutkey_id))
        elif subset.endswith("2"):
            if all_db == "Y" and gene_all_db == "Y": positives.append((all_db, sc_val, drug_name_id, mutkey_id))
            elif gene_all_db == "Y": negatives.append((all_db, sc_val, drug_name_id, mutkey_id))  
            if all_db == "N": 
                negatives_all.append((hc_db, sc_val, drug_name_id, mutkey_id))
    input_fh.close()

    if subset.startswith("all"):
        negatives_rand = negatives
    elif subset.startswith("eq"):
        negatives_rand = random.sample(negatives, min(len(negatives), len(positives)))
    elif subset.startswith("frac"):
        negatives_rand = random.sample(negatives, int(len(negatives)*frac))
    elif subset.startswith("sel"):
        negatives_rand = random.sample(negatives_all, len(negatives))
    else:
        print("Provide correct subset name")
        sys.exit()


    for (db, val, drug_name_id, mutkey_id) in positives:
        pos_sel.add((drug_name_id, mutkey_id))

    for (db, val, drug_name_id, mutkey_id) in negatives_rand:
        neg_sel.add((drug_name_id, mutkey_id))

    return pos_sel, neg_sel

def calcStats(score, input_dir, revord=1, pos_sel=None, neg_sel=None, thrs=None):
    positives = []
    negatives = []

    input_fn = input_dir + score  + ".txt"
    if not os.path.exists(input_fn):
        return None

    thrs_tmp = set([])


    input_fh = open(input_fn)
    lines = input_fh.readlines()
    input_fh.close()
    k = len(lines)
    for i in range(k):
        tokens = lines[i].split()
        if len(tokens) < 4: continue
        drug_name_id = tokens[0]
        mutkey_id = tokens[1]
        all_db = tokens[2]
        gene_all_db = tokens[4]
        gene_hc_db = tokens[5]
        sc_val = float(tokens[6])
        if (drug_name_id, mutkey_id) in pos_sel:
            positives.append(("Y", sc_val))
        elif (drug_name_id, mutkey_id) in neg_sel:
            negatives.append(("N", sc_val))
        if thrs == None:
            thrs_tmp.add(sc_val)

    if thrs == None:
        thrs = thrs_tmp
    n = len(negatives)
    p = len(positives)

    res_list = sorted(negatives + positives, key=lambda (db, sc_val):-1*revord*sc_val)
    thrs_sorted = sorted(thrs, key=lambda sc_val:-1*revord*sc_val)

    tp, fp, tn, fn = [],[],[],[]
    prec, recall = [],[]
    tpc, fpc, tnc, fnc = 0,0,n,p

    thrs_used = []
    i = 0
    sc_val = res_list[i]

    stats_map = {}

    for j in range(len(thrs_sorted)):
        thr_val = thrs_sorted[j]
        while i < n+p and sc_val >= thr_val * revord:
            db, sc_val = res_list[i]
            if db == "Y":
                tpc += 1; fnc -= 1
            else:
                fpc += 1; tnc -= 1  
            i += 1
      #  print(score, n,p, n+p, j,i, "x", tpc, fpc, tnc, fnc)
        if i > 0:
            thrs_used.append(thr_val)
            tp.append(tpc)
            fp.append(fpc)
            tn.append(tnc)
            fn.append(fnc)
            prec_tmp, recall_tmp = float(tpc) / i, float(tpc) / p
            prec.append(prec_tmp)
            recall.append(recall_tmp) 
            stats_map[thr_val] = (prec_tmp, recall_tmp)          

    return stats_map

if __name__ == '__main__':

    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    cmp_dir = parameters["RESULTS_CMP_DIR"]
    dataset = parameters["D"]
    frac = float(parameters["FRAC"])
    reps_tmp = int(parameters.get("RAND", "1"))

    assocs_dir = cmp_dir + "/assocs/"
    ensure_dir(assocs_dir)
    ensure_dir(cmp_dir)

    scores_cmp = set(parameters.get("SCORES", "ws,or,mi,lh,r-tgh").split(","))

    scores_list = []
    for score_fn in os.listdir(assocs_dir):
        score = score_fn[:-4]
        if not score in scores_cmp:
            continue
        scores_list.append(score)

    subsets = parameters.get("SUBSETS", "eq1,eq3").split(",")
    print("SUBSETS", subsets)
    
    for subset in subsets:
        if subset.startswith("all"): reps = 1
        else: reps = reps_tmp

        stats_dir = cmp_dir + "/stats_"+subset+"/"
        ensure_dir(stats_dir)

        for score in scores_list:
            stats_score_dir = cmp_dir + "/stats_"+subset+"/" + score
            ensure_dir(stats_score_dir)
            for i in range(reps):
                tls = []

                pos_sel, neg_sel = selectAssocs(assocs_dir + score  + ".txt", subset, frac)

                print("method: ",subset, " rep:", i, "score:", score, "pos:", len(pos_sel), "neg:", len(neg_sel))
                rev_order = 1
                if score.startswith("pv-") or score.count("+") > 0:
                    rev_order = -1

                stats_map = calcStats(score, assocs_dir, revord=rev_order, pos_sel=pos_sel, neg_sel=neg_sel)

                thrs_sorted = sorted(stats_map, key=lambda thr:-1*rev_order*thr)

                for thr in thrs_sorted:
                    (prec_tmp, recall_tmp) = stats_map[thr]
                    
                    tl = "\t".join([str(thr), str(prec_tmp), str(recall_tmp)])
                    tls.append(tl + "\n")

                output_fh = open(stats_score_dir + "/"+str(i) + ".txt", "w")
                for tl in tls:
                    output_fh.write(tl)
                os.fsync(output_fh.fileno())
                output_fh.close()
                
    print("tb2_save_stats_lists_rep.py: Finished.")
