import sys
import os
import time
import math
import shutil
import random
import multiprocessing
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_utils import ensure_dir

def computeAUC(prec_v, recall_v):

    auc = 0.0
    n = len(prec_v)
    for i in range(1, n, 1):
        auc = auc + 0.5*(prec_v[i] + prec_v[i-1])*(recall_v[i]-recall_v[i-1])
    #print("p", prec_v)
    #print("r", recall_v)
    return auc

def avg(list):
    if len(list)== 0:
        return 0.0
    sum = 0
    for elm in list:
        sum += elm
    return sum/(len(list)*1.0)


if __name__ == '__main__':

    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    cmp_dir = parameters["RESULTS_CMP_DIR"]
    dataset = parameters["D"]
    frac = float(parameters["FRAC"])
    reps_tmp = int(parameters.get("RAND", "1"))

    assocs_dir = cmp_dir + "/assocs/"
    ensure_dir(assocs_dir)
    ensure_dir(cmp_dir)

    scores_list = []
    for score_fn in os.listdir(assocs_dir):
        score = score_fn[:-4]
        scores_list.append(score)

    subsets = parameters.get("SUBSETS", "eq1,eq3").split(",")
    print("SUBSETS", subsets)
    
    for subset in subsets:
        if subset.startswith("all"): reps = 1
        elif subset.startswith("sel"): reps = reps_tmp
        else: reps = reps_tmp

        output_dir = cmp_dir + "/stats_"+subset+"_rands/"
        ensure_dir(output_dir)

        results_map = []
        for i in range(reps):
            results_map.append({})
        results_mean_map = {}

        for score in scores_list:
            stats_score_dir = cmp_dir + "/stats_"+subset+"/" + score + "/"
            auc_v = []

            tls = []

            for i in range(reps):
                input_fn = stats_score_dir + "/"+str(i) + ".txt"
                if not os.path.exists(input_fn):
                    continue

                prec_v = [1.0]
                recall_v = [0.0]
                input_fh = open(input_fn)
                lines = input_fh.readlines()
                for line in lines:
                    score_str, prec_str, recall_str = line.split()
                    prec_v.append(float(prec_str))
                    recall_v.append(float(recall_str))
                input_fh.close()

                auc = computeAUC(prec_v, recall_v)
                auc_v.append(auc)
                results_map[i][score] = auc
                tl = str(auc)
                tls.append(tl)
            score_mean = avg(auc_v)
            results_mean_map[score] = score_mean

            output_fh = open(output_dir + "/" + score + ".txt", "w")
            for tl in tls:
                output_fh.write(tl + "\n")
            output_fh.close()

        scores_list_sorted = sorted(scores_list, key=lambda sc:results_mean_map[sc])

        output_fh = open(cmp_dir + "/" + subset + ".txt", "w")
        output_fh.write("\t".join(scores_list_sorted) + "\n")
        
        for i in range(len(results_map)):
            tl = ""
            for score in scores_list_sorted:
                tl += str(results_map[i].get(score,0.0)) + "\t"
            tl = tl[:-1] + "\n"
            output_fh.write(tl)
        output_fh.close()

    print("cmp2_comp_stats_rep.py: Finished.")
