import sys
import os
import copy
import multiprocessing
from multiprocessing import Pool


sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_ann_utils import readClusterGeneIDs
from drsoft.utils.gwamar_muts_utils import *
from drsoft.utils.gwamar_res_utils import compareProfiles
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.utils.gwamar_res_io_utils import *

def getResSet(dr_profile, inc_q=False):
    res_set = set([])
    for i in range(len(dr_profile)):
        if dr_profile[i] == "R" or (inc_q == True and dr_profile[i] == "?"):
            res_set.add(i)
    return res_set

def getBinSet(dr_profile):
    bin_set = set([])
    for i in range(len(dr_profile)):
        if dr_profile[i] != "0" and dr_profile[i] != "?":
            bin_set.add(i)
    return bin_set   

if __name__ == '__main__':
    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    WORKERS = 1#min(int(parameters["W"]), multiprocessing.cpu_count())
    
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    analysis_comb_dir = parameters["RESULTS_A_COMB_S_SEL_DIR"]
    analysis_comp_dir = parameters["RESULTS_A_COMP_S_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    comp_dir = parameters["COMP_DATA"]

    ensure_dir(comp_dir)
    drug_name = "Rifampicin"

    input_fn = analysis_comb_dir + drug_name + ".txt"
    input_fh = open(input_fn)
    lines = input_fh.readlines()
    line0 = lines[0]
    dr_profile = line0.strip().split("\t")[0]

    dr_set = getResSet(dr_profile, inc_q=False)

    gene_muts = {}
    
    for line in lines[1:]:
        line = line.strip()
        tokens = line.split("\t")
        bin_profile = tokens[0]
        mut_descs = tokens[-1]
        mut_set = getBinSet(bin_profile)
        if len(mut_set & dr_set) == 0:
            continue
        for mut_desc in mut_descs.split(";"):
            gene_id, cluster_id, gene_name, position, ref_state, mut_states = mut_desc.split()

            if not gene_name in gene_muts:
                gene_muts[gene_name] = []
            if int(position) < 0:
                continue
            gene_muts[gene_name].append(str(position) + "\t" + ref_state + "\t" + mut_states)
            
    input_fh.close()
    tls = []

    tls.append("%" + parameters["D"] +"\n")

    for gene_name in sorted(gene_muts):
        tls.append(">" + gene_name + "\n")
        for mut_desc in sorted(gene_muts[gene_name]):
            tls.append(mut_desc + "\n")

    output_fn = comp_dir + "/" +parameters["D"] + ".txt"
    output_fh = open(output_fn, "w")
    for tl in tls:
        output_fh.write(tl)
    output_fh.close()
