import os
import sys
import filecmp

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_res_io_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.structs.resistance_profile import ResistanceProfile
from drsoft.utils.gwamar_utils import ensure_dir

def readMutsList(input_fh):
    uncovered = set([])
    covered_map = {}
    muts_list = []
    gene_name = ""

    lines = input_fh.readlines()
    for line in lines:
        line = line.strip()
        line = line.split("#")[0].strip()

        tokens = line.split()
        if len(tokens) == 1:
            uncovered.add(int(tokens[0]))        
        elif len(tokens) == 2:
            covered_map[int(tokens[0])] = int(tokens[1])
        elif len(tokens) >= 5:
            mut_id_str, g_id, g_name, p_str, ref_state, mut_state = tokens
            position = int(p_str)
            mut_id = int(mut_id_str)
            muts_list.append((mut_id, g_name, position, ref_state, mut_state))

    return uncovered, covered_map, muts_list


gene_shift = {"rpoA":50, "rpoB":300, "rpoC": 550}

gene_length = {"rpoA": 347, "rpoB": 1172, "rpoC": 1314}

x_offset = 10

drug_name = "Rifampicin"
colors = {0: "blue", 1: "orange", 2: "green", 3: "purple", 4: "violet", 5: "brown"}
occupation = {}
occupation_rev = {}
cover_color_map = {}

if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()

    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    stats_dir = parameters["STATS_DIR"]
    latex_dir = parameters["LATEX_DIR"]
    comp_dir = parameters["COMP_DATA"]
    analysis_comb_dir = parameters["RESULTS_A_COMB_S_SEL_DIR"]
    analysis_comp_dir = parameters["RESULTS_A_COMP_S_DIR"]

    ensure_dir(stats_dir)
    ensure_dir(latex_dir)

    input_fh = open(input_dir + "/strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    input_fh = open(results_dir + "/res_profiles.txt")
    res_profiles_list = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()

    input_fh = open(analysis_comp_dir + drug_name + "/relations_sep.txt")
    uncovered, covered_map, muts_list = readMutsList(input_fh)
    input_fh.close()

    res_profiles_sorted = sorted(res_profiles_list, key=lambda drp: -len(drp.getKnown()))

    tls = []
    tls.append("\\documentclass[10pt]{article}\n")
    tls.append("\\usepackage{tikz}\n")
    tls.append("\\usepackage[landscape]{geometry}\n")
    tls.append("\\geometry{a4paper,left=2mm,right=2mm}\n")
    tls.append("\\begin{document}\n")
    tls.append("\\begin{tikzpicture}[x=0.5pt,y=0.5pt]\n")

    for gene_name in ["rpoA", "rpoB", "rpoC"]:
        gene_len = gene_length[gene_name]
        tls.append("\\draw [color=black] ("+str(x_offset)+","+str(gene_shift[gene_name])+") -- ("+str(x_offset + gene_len)+","+str(gene_shift[gene_name])+");\n")
        tls.append("\\node [right] at (" + str(0) + "," + str(gene_shift[gene_name]-40) + ") {" + gene_name + "};\n")

        for p in range(0, gene_len+99, 100):
            px = min(p, gene_len)
            x1 = x_offset + px
            y1 = gene_shift[gene_name]
            x2 = x_offset + px
            y2 = gene_shift[gene_name] - 10
            tls.append("\\draw [color=black] ("+str(x1)+","+str(y1)+") -- ("+str(x2)+","+str(y2)+");\n")
            if gene_len - px < 40 and gene_len != px:
                continue
            tls.append("\\node [right] at (" + str(px-5) + "," + str(gene_shift[gene_name]-20) + ") {" + str(px) + "};\n")



    for (mut_id, gene_name, position, ref_state, mut_state) in muts_list:
        if mut_id in uncovered:
            color = "red"
        else:
            if mut_id in covered_map:
                cover_id = covered_map[mut_id]
                if not cover_id in cover_color_map:
                    cover_color_map[cover_id] = colors.get(len(cover_color_map), "pink")
                color = cover_color_map[cover_id]
            else:
                color = "black"

        if not (gene_name, position) in occupation:
            occupation[(gene_name, position)] = 0
        occupation[(gene_name, position)] += 1
        var_count = occupation[(gene_name, position)]

        x1 = x_offset + position
        y1 = gene_shift[gene_name] + var_count*10
        x2 = x_offset + position
        y2 = gene_shift[gene_name] + var_count*10 + 10
        tl = "\\draw [color="+color+"] ("+str(x1)+","+str(y1)+") -- ("+str(x2)+","+str(y2)+");\n"
        tls.append(tl)

    
  # tls.append(tl)

    tls.append("\\end{tikzpicture}\n")
    tls.append("\\end{document}\n")

    output_fn = analysis_comp_dir + drug_name + "/comp_rpoB_rels.tex"

    output_fh = open(output_fn, "w")
    for tl in tls:
        output_fh.write(tl)
    output_fh.close()

    os.chdir(analysis_comp_dir + drug_name)
    command = "pdflatex " + output_fn
    os.system(command)
