import sys
from collections import deque

from drsoft.modelling.model_logic import *
from drsoft.utils.gwamar_stat_utils import *
from drsoft.utils.gwamar_generator_utils import generateSumProfiles


def colorEdges(tree, alg=0, overwrite=False):
    if alg == 0: # color all edges coming to resistant nodes
        for node in list(tree.leaves) + tree.inner_nodes_bu:
            if node.complete_res_state == "R":
                if overwrite == True or node.ecolor == None:
                    node.ecolor = "b"
            else:
                if overwrite == True or node.ecolor == None:
                    node.ecolor = "w"
    elif alg == 1: # color only edges with S->R
        for node in list(tree.leaves) + tree.inner_nodes_bu:
            if node.complete_res_state == "R" and (node.parent == None or node.parent.complete_res_state == "S"):
                if overwrite == True or node.ecolor == None:
                    node.ecolor = "b"
            else:
                if overwrite == True or node.ecolor == None:
                    node.ecolor = "w"
    return None

def calcN(tree):
    for node in tree.nodes.values():
        node.n = node.subleaves_count
    return tree.root.n

def calcMaxN(tree):
    for node in tree.leaves:
        node.maxn = 1

    for node in tree.inner_nodes_bu:
        sum_tmp = 0
        any_more_1 = False
        all_leaves = True
        for ch_node in node.children:
            sum_tmp += ch_node.maxn
            if ch_node.maxn > 1:
                any_more_1 = True
            if not ch_node.is_leaf():
                all_leaves = False 
        if any_more_1:
            node.maxn = sum_tmp
        elif not all_leaves:
            node.maxn = sum_tmp
        else:
            node.maxn = len(node.children)-1

    return tree.root.maxn

def calcWnMAT(tree, n1):
    for node in tree.leaves:
        node.vn = {0:1, 1:0}

    for node in tree.inner_nodes_bu:
        node.vn = {0: 1}

        children_list = node.children
        k = len(children_list)

        for n in range(1, node.maxn+1, 1):
            node.vn[n] = 0

            for p in range(min(n+1, k)):
                for gl in generateSumProfiles2(k, p, ub=[1]*k):
                    ubl = []
                    for i in range(k):
                        node_i = children_list[i]
                        
                        if gl[i] == 1: ubl.append(0)
                        else: ubl.append(node_i.maxn)

                    for nv in generateSumProfiles2(k, n-p, ub=ubl):
                        print("x", node.node_id, node.maxn, n, gl, nv)
                        c = 1
                        for i in range(k):
                            if gl[i] == 0:
                                c *= children_list[i].vn[nv[i]]
                        node.vn[n] += c
    for node in tree.inner_nodes_bu:
        print(node.node_id, len(node.children), node.vn)
    return tree.root.vn

def calcPnmkMAT(tree, n1, n2):
    mem = {}

    for node in tree.leaves:
        node.bnmk = {}
        mem[node.topos] = {}
        node.bnmk[(0,0,0)] = 1
        mem[node.topos][(0,0,0)] = 1

        for m in range(node.n+1):
            node.bnmk[(node.n, m, m)] = node.vn[m]
            mem[node.topos][(node.n, m, m)] = node.vn[m]
            for k in range(m):
                node.bnmk[(node.n, m, k)] = 0
                mem[node.topos][(node.n, m, k)] = 0

    for node in tree.inner_nodes_bu:
        if node.topos in mem:
            continue
        node.bnmk = {}
        mem[node.topos] = {}
        node.bnmk[(0,0,0)] = 1
        mem[node.topos][(0,0,0)] = 1

        for m in range(node.n+1):
            node.bnmk[(node.n, m, m)] = node.vn[m]
            mem[node.topos][(node.n, m, m)] = node.vn[m]
            for k in range(m):
                node.bnmk[(node.n, m, k)] = 0
                mem[node.topos][(node.n, m, k)] = 0

        children_list = node.children
        r = len(node.children)

        for n in range(0, min(n1+1, node.n), 1):
            print(node.node_id, r, node.n, n)
            for m in range(0, min(n2, n)+1, 1):
                for k in range(max(0, n+m-node.n), min(m, node.n)+1, 1):

                    if r == node.n:
                        bnmk_tmp = binom(node.n, m)*binom(m, k)*binom(node.n-m, n-k)            
                    else:
                        ubn = []
                        for i in range(r):
                            ubn.append(children_list[i].n)
                        bnmk_tmp = 0

                        for nv in generateSumProfiles2(r, n, ub=ubn):
                            for mv in generateSumProfiles2(r, m, ub=ubn):
                                ubk = []
                                lbk = []
                                for i in range(r):
                                    lbk.append(max(0, nv[i]+mv[i]-children_list[i].n))
                                    ubk.append(min(mv[i], nv[i]))

                                for kv in generateSumProfiles2(r, k, lb=lbk, ub=ubk):
                                    c = 1
                                    for i in range(r):
                                        if nv[i] >= mv[i]:
                                            c *= mem[children_list[i].topos][(nv[i], mv[i], kv[i])]
                                        else:
                                            c *= mem[children_list[i].topos][(mv[i], nv[i], kv[i])]
                                    bnmk_tmp += c
                    node.bnmk[(n,m,k)] = bnmk_tmp
                    mem[node.topos][(n,m,k)] = bnmk_tmp
    tree.root.bnmk = mem[tree.root.topos]
    return tree.root.bnmk


