import os
import sys
from decimal import Decimal

from collections import deque

from drsoft.modelling.model_logic import *
from drsoft.utils.gwamar_stat_utils import *
from drsoft.utils.gwamar_bin_utils import *

from drsoft.utils.gwamar_generator_utils import generateSumProfiles

def colorEdges(tree, alg=0, overwrite=False):
    if alg == 0: # color all edges coming to resistant nodes
        for node in list(tree.leaves) + tree.inner_nodes_bu:
            if node.complete_res_state == "R":
                if overwrite == True or node.ecolor == None:
                    node.ecolor = "b"
            else:
                if overwrite == True or node.ecolor == None:
                    node.ecolor = "w"
    elif alg == 1: # color all edges with S->R
        for node in list(tree.leaves) + tree.inner_nodes_bu:
            if node.complete_res_state == "R" and (node.parent == None or node.parent.complete_res_state == "S"):
                if overwrite == True or node.ecolor == None:
                    node.ecolor = "b"
            else:
                if overwrite == True or node.ecolor == None:
                    node.ecolor = "w"
    return None

def calcFastBC(tree):
    for node in tree.leaves: node.bc = 0
    for node in tree.inner_nodes_bu:
        node.bc = 0
        for node_j in node.children:
            if node_j.ecolor == "b":
                node.bc += 1
    return tree.root.bc

def calcFastN(tree, w2=False):
    for node in tree.leaves: node.maxn = 1

    if w2:
        for node in tree.inner_nodes_bu:
            sum_tmp = 0
            all_leaves = True
            for ch_node in node.children:
                sum_tmp += max(1, ch_node.maxn)
                if not ch_node.is_leaf():
                    all_leaves = False 
            if all_leaves:
                node.maxn = len(node.children)-1
            else:
                node.maxn = sum_tmp
    else:
        for node in tree.inner_nodes_bu:
            node.maxn = 0
            for ch_node in node.children:
                node.maxn += max(ch_node.maxn, 1)

    return tree.root.maxn

def calcFastP(tree, w2=False):
    for node in tree.leaves: 
        node.maxp0 = 0
        if node.ecolor == "b": node.maxp = 1
        else: node.maxp = 0

    if w2:
        for node in tree.inner_nodes_bu:
            if node.res_subleaves_count == node.subleaves_count:
                node.maxp = node.res_subleaves_count-1
            else:
                node.maxp = node.res_subleaves_count
    else:
        for node in tree.inner_nodes_bu:
            node.maxp = node.res_subleaves_count

    return tree.root.maxp

def calcFastWnMAT(tree, maxn_t=None, opt=False, w2=True):
    for node in tree.leaves:
        node.count_map = {}                             
        node.count_map[0] = 1

    for node in tree.inner_nodes_bu:
        node.count_map = {}
        children_list = node.children
        k = len(node.children)

        maxn = min(maxn_t, node.maxn)

        for nt in range(maxn+1):
            if w2: maxr = min(nt, k-1, node.maxn)
            else: maxr = min(nt, k, node.maxn)
            
            if opt and (k+1 == node.subnodes_count):
                if w2 and k == nt: wnm = 0
                else: wnm = binom(k, nt)
            else:
                wnm = 0
            
                for r in range( maxr+1):
                 #   if w2 == False and r==k: wnm += 1
                    for gain_profile in generateSumProfiles2(k, r, ub=[1]*k,lb=[0]*k):
                        gain_bound = []
                        for j in range(k): 
                            if gain_profile[j] == 1: 
                                gain_bound.append(0)
                            else:
                                gain_bound.append(node.maxn)
                        for gain_subprofile in generateSumProfiles2(k, nt-r, ub=gain_bound):
                            wnm_tmp = 1
                            for j in range(k):
                                if gain_profile[j] == 0:
                                    node_j = node.children[j]
                                    wnm_tmp *= node_j.count_map.get(gain_subprofile[j], 0)
                                    if wnm_tmp == 0:
                                        break    
                            wnm += wnm_tmp
            node.count_map[nt] = wnm
    return tree.root.count_map

def calcFastRP(tree):
    # r - total number of 0->1 changes 
    # p - total number of 0->1 changes on black edges
    r,p = 0,0
    for node in tree.inner_nodes_bu:
        for node_j in node.children:
            if node.complete_mut_state == '0' and node_j.complete_mut_state == "1":
                r += 1
                if node_j.ecolor == "b":
                    p += node_j.res_subleaves_count
    return r,p

def calcFastBpnMAT(tree, maxp_t, maxn_t, opt=False, w2=True, mem={}):
    mem = {}

    for node in tree.leaves:
        mem[node.topos] = {}
        mem[node.topos][(0,0)] = 1
        
    for node in tree.inner_nodes_bu:
        maxn = min(maxn_t, node.maxn)
        maxp = min(node.maxp, maxp_t)

        if node.topos in mem and (maxp, maxn) in mem[node.topos]:
            continue
        if not node.topos in mem:
            mem[node.topos] = {}
        
        children_list = list(node.children)
        k = len(children_list)

        mem[node.topos][(0,0)] = 1

        for nt in range(1, maxn + 1, 1): # liczba wybranych
            maxp = min(node.maxp, maxp_t)
            if (maxp, nt) in mem[node.topos]:
                continue
            for pt in range(maxp + 1):     # liczba pokrytych
                if (pt, nt) in mem[node.topos]:
                    continue
                if opt and (k+1 == node.subnodes_count):
                    if w2 and k == nt: wnm = 0
                    else: wnm = binom(node.bc, pt) * binom(k-node.bc, nt-pt)
                else:
                    wnm = 0
                    if w2: maxr = min(k-1, nt)
                    else: maxr = min(k, nt)
                    for r in range(maxr + 1):
                        for gain_r_profile in generateSumProfiles2(k, r, ub=[1]*k,lb=[0]*k):
                            if w2 and sum(gain_r_profile) == k:
                                continue

                            ur = 0 # liczba trafionych
                            gain_r_bound = []
                            for j in range(k):
                                node_j = children_list[j]
                                if gain_r_profile[j] == 1:
                                    gain_r_bound.append(0)
                                    if node_j.ecolor == "b":
                                        ur += node_j.res_subleaves_count
                                elif gain_r_profile[j] == 0:
                                    gain_r_bound.append(min(node_j.maxn, nt-r))

                            for gain_r_subprofile in generateSumProfiles2(k, nt-r, ub=gain_r_bound):
                                gain_p_bound = []
                                for j in range(k):
                                    node_j = children_list[j]
                                    if gain_r_profile[j] == 1:
                                        gain_p_bound.append(0)
                                    else:
                                        gain_p_bound.append(min(node_j.maxp, pt-ur))

                                for gain_u_subprofile in generateSumProfiles2(k, pt-ur, ub=gain_p_bound):
                                    wnm_tmp = 1
                                    for j in range(k):
                                        if gain_r_profile[j] == 0:
                                            node_j = children_list[j]
                                            ptmp, ntmp = gain_u_subprofile[j], gain_r_subprofile[j]
                                            wnm_tmp *= mem[node_j.topos].get((ptmp, ntmp), 0)
                                            if wnm_tmp == 0:
                                                break
                                    wnm += wnm_tmp

                mem[node.topos][(pt,nt)] = wnm
    return mem
    
def calcBinProfilesXCCT(bin_profiles, drp, trees_list, norm=False, opt=True, w2=True, mem={}):
    scores = {}
    
    for bin_profile_id, bin_profile in bin_profiles.iteritems():
        scores[bin_profile_id] = []
    
    for tree in trees_list:
        tree.setObservedResProfile(drp.full_profile)
        tree.calcInnerNodesBU()
        calcSubleavesResCount(tree)
        calcSubnodesCount(tree)

        if len(drp.getResistant()) == 0:
            continue
        
        newtree = removeUnResLeaves(tree)
        newtree.calcInnerNodesBU()
        newtree.setObservedResProfile(drp.full_profile)
        calcSubleavesResCount(newtree)
        calcSubnodesCount(newtree)
        calcDetParsResModel(newtree)
        colorEdges(newtree, alg=0, overwrite=False)
        computeTopos(newtree, r=True)

        maxn = 0
        maxp = 0

        n_dist = {}
        
        for bin_profile_id, gen_bin_profile in bin_profiles.iteritems():
            for bin_profile in specBinProfiles(gen_bin_profile):
                newtree.setObservedMutProfile(bin_profile)
                calcSubleavesMutCount(newtree)
                calcDetParsMutModel(newtree)
                r,p = calcFastRP(newtree)
                maxn = max(maxn,r)
                maxp = max(maxp,p)
                if not r in n_dist:
                    n_dist[r] = 0.0
                n_dist[r] += 1.0
        
        calcFastBC(newtree)
        nx = calcFastN(newtree, w2=w2)
        px = calcFastP(newtree, w2=w2)

        t0 = time.time()
        wn_mat = calcFastWnMAT(newtree, maxn, opt=opt, w2=w2)
        t1 = time.time()
        
        mem = calcFastBpnMAT(newtree, maxp, maxn, opt=opt, w2=w2, mem=mem)
        bpn_mat = mem[newtree.root.topos]

        test = {}
        for (pt,nt) in bpn_mat:
            if not nt in test: 
                test[nt] = 0
            test[nt] += bpn_mat[(pt,nt)]

        t2 = time.time()

        for bin_profile_id, gen_bin_profile in bin_profiles.iteritems():
            score = 0.0
            for bin_profile in specBinProfiles(gen_bin_profile):
                newtree.setObservedMutProfile(bin_profile)
                calcSubleavesMutCount(newtree)
                calcDetParsMutModel(newtree)
                
                r,p = calcFastRP(newtree)

                bs = wn_mat[r]
                for pt in range(p):
                    bs -= bpn_mat[(pt, r)]

                cct_score = Decimal(bs) / Decimal(wn_mat[r])
                try:
                    if norm:
                        score += -math.log(float(cct_score)*n_dist[r])
                    else:
                        score += -math.log(float(cct_score))

                except:
                    pass
                 
            scores[bin_profile_id].append(score)

    scores_mean = {}
    
    for bin_profile_id in scores:
        scores_mean[bin_profile_id] = mean(scores[bin_profile_id])

    return scores_mean, mem


def readXCCTMem(mem_fn):
	if not os.path.exists(mem_fn):
		return {}
	mem = {}

	input_fh = open(mem_fn)
	lines = input_fh.readlines()
	input_fh.close()
	topo = ""

	for line in lines:
		tokens = line.split()
		if len(tokens) < 1: 
			continue
		elif len(tokens) == 1:
			topo = tokens[0]
			if not topo in mem:
				mem[topo] = {}
		else:		
			k = int(tokens[0])
			n = int(tokens[1])
			b = int(tokens[2])
			mem[topo][(k,n)] = b

	return mem

def saveXCCTMem(mem, mem_fn):
	output_fh = open(mem_fn, "w")
	topo_sorted = sorted(mem.keys(), key=lambda topo:(len(topo), topo))
	for topo in topo_sorted:
		output_fh.write(topo + "\n")
		for (k, n) in sorted(mem[topo], key=lambda (k,n):(n,k)):
			b = mem[topo][(k,n)]
			output_fh.write(str(k) + "\t" + str(n) + "\t" + str(b) + "\n")
	output_fh.close()

