import sys
import os
import copy
import multiprocessing
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_progress_utils import GWADRIBProgress
from drsoft.utils.gwamar_ann_utils import readClusterGeneIDs
from drsoft.utils.gwamar_muts_utils import *
from drsoft.utils.gwamar_res_utils import compareProfiles
from drsoft.utils.gwamar_res_io_utils import readResistanceProfiles
from drsoft.utils.gwamar_muts_io_utils import readDrugResRel
from drsoft.utils.gwamar_strains_io_utils import readStrains

def loadDrugResMuts(input_fh, cluster_ids_rev={}):

    drug_targets = {}
    lines= input_fh.readlines();
    for line in lines:
        line = line.strip()
        if not line: break
        tokens_hash = line.split("#")
        if len(tokens_hash) > 1: line = tokens_hash[0]
        tokens = line.split()
        
        if len(tokens) < 5: continue
        
        drug_name = tokens[0]
        if not drug_name in drug_targets:
            drug_targets[drug_name] = {}

        gene_id = tokens[2]
        position = int(tokens[3])
        ref_states = tokens[4]
        mut_states = tokens[5]
        
        cluster_id =  cluster_ids_rev.get(gene_id, gene_id)
       # print(line, cluster_id)

        if not cluster_id in drug_targets[drug_name]:
            drug_targets[drug_name][cluster_id] = {}
        if not position in drug_targets[drug_name][cluster_id]:
            drug_targets[drug_name][cluster_id][position] = set([])
        drug_targets[drug_name][cluster_id][position].add((ref_states, mut_states))
    return drug_targets

def loadDrugResRel(input_fh, cluster_ids_rev={}):

    drug_targets = {}
    lines= input_fh.readlines();
    for line in lines:
        line = line.strip()
        if not line: break
        tokens_hash = line.split("#")
        if len(tokens_hash) > 1: line = tokens_hash[0]
        tokens = line.split()
        
        print(tokens)
        
        if len(tokens) < 5: continue
        
        drug_name = tokens[0]
        if not drug_name in drug_targets:
            drug_targets[drug_name] = {}
        gene_id = tokens[2]
        position = int(tokens[3])

        cluster_id =  cluster_ids_rev.get(gene_id, gene_id)

        start = position
        end = position
        
        if not cluster_id in drug_targets[drug_name]:
            drug_targets[drug_name][cluster_id] = [] 
        drug_targets[drug_name][cluster_id].append((start, end))
    return drug_targets

def readRanksMap(input_fh):
    ranks_map = {}
    for line in input_fh.readlines():
        tokens = line.split()
        if len(tokens) < 3: continue
        bin_profile_id = tokens[0]
        rank = (float(tokens[1]) + float(tokens[2])) / 2
        ranks_map[bin_profile_id] = rank
    return ranks_map

if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    dataset = parameters["DATASET"]
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    ranks_dir = parameters["RESULTS_RANKS_DIR"]
    analysis_dir = parameters["RESULTS_ANALYSIS_DIR"]
    exp_prefix = parameters["EXP_PREFIX"]
    
    input_fh = open(input_dir + "/strain_profiles_ord.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    m_type = "P"
    
    score_params = ["tscore", "mi", "or", "wsupport"]
    score_params += ["tmow"]

    input_fh = open(input_dir + "/res_profiles.txt")
    drug_res_profiles = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    input_fh = open(input_dir + 'bin_profiles_' + m_type + '_det.txt')
    bin_profile_details = readBinProfileDetails(input_fh, mut_type="P", out_format="tuple")
    input_fh.close()
    
    if os.path.exists(input_dir + "/cluster_gene_ref_ids.txt"):
        cluster_fh = open(input_dir + "/cluster_gene_ref_ids.txt")
        cluster_ids, cluster_ids_rev = readClusterGeneIDs(cluster_fh, gene_ref_ids=True)
        cluster_fh.close()
    else:
        cluster_ids = {} 
        cluster_ids_rev = {}
    
    input_fh = open(input_dir + "/drug_res_S_G_A.txt")
    drug_res_rel = readDrugResRel(input_fh, cluster_ids_rev)
    input_fh.close()
    
    input_fh = open(input_dir + "/drug_res_S_H.txt")
    drug_res_conf_db = loadDrugResMuts(input_fh, cluster_ids_rev)
    input_fh.close()
    
    input_fh = open(input_dir + "/drug_res_S_A.txt")
    drug_res_all_db = loadDrugResMuts(input_fh, cluster_ids_rev)
    input_fh.close()

    text_lines_best = []
    
    text_line = "drug_name" +"\t" 
    text_line += "gene_id" + "\t" 
    text_line += "gene_name" + "\t"
    text_line += "position" + "\t" 
    for score in score_params:
        text_line += str(score) + "\t" 

    text_line += "ref_states_broad" + "\t"
    text_line += "non_ref_states_broad" + "\t"
    text_line += "ref_states_all" + "\t"
    text_line += "non_ref_states_all" + "\t"
    text_line += "ref_states_high" + "\t"
    text_line += "non_ref_states_high" + "\t"

    text_line += "all" + "\t"
    text_line += "high" + "\n"
    text_lines_best.append(text_line)

    
    for drug_res_profile in drug_res_profiles:
        drug_name = drug_res_profile.drug_name
        drug_rel_muts = drug_res_rel.get(drug_name, {})

        bin_profiles_res_rel = selectResRelBinProfiles(bin_profile_details, drug_rel_muts, cluster_ids, cluster_ids_rev)

        input_fn = scores_dir + "/" + drug_name  + "/tscore.txt"
        if not os.path.exists(input_fn) or os.path.getsize(input_fn) < 10:
            print("NOT EXIST: ", input_fn)
            continue
            
        input_fh = open(input_fn)
        bin_profiles_sorted = readBinProfilesSorted(input_fh, False)
        input_fh.close()

        ranks_map = {}
        
        for score in score_params:
            input_fn = ranks_dir + "/" + drug_name  + "/" + score + ".txt"
            if not os.path.exists(input_fn) or os.path.getsize(input_fn) < 10:
                print("NOT EXIST: ", input_fn)
                continue
            input_fh = open(input_fn)
            ranks_map[score] = readRanksMap(input_fh)
            input_fh.close()   
                
        n = len(bin_profiles_sorted)
        if n == 0: exit()
    
        used_gene_pos = set([])
        for i in range(n):
            bin_profile_id, bin_profile_score = bin_profiles_sorted[i]
            for (gene_id, cluster_id, gene_name, position, res_states_txt, mut_states_txt) in bin_profiles_res_rel.get(bin_profile_id, []):
                text_line = drug_name +"\t"
                text_line += gene_id + "\t"
                text_line += gene_name + "\t"
                text_line += str(position) + "\t"
                
                for score in score_params:
                    try:
                        text_line += str(ranks_map[score][bin_profile_id]) + "\t"
                    except:
                        print(drug_name, score, bin_profile_id)
                text_line += str(res_states_txt) + "\t"
                text_line += str(mut_states_txt) + "\t"
                
                src_muts = set([])
                dst_muts = set([])
                src_muts_txt = ""
                dst_muts_txt = ""
                
                for (mut_src, mut_dst) in drug_res_all_db.get(drug_name, {}).get(cluster_id, {}).get(position, set([])):
                    src_muts.add(mut_src)
                    dst_muts.add(mut_dst)
                
                if len(src_muts) > 1 and "X" in src_muts: src_muts.remove("X")
                
                for mut_src in src_muts:
                    src_muts_txt += mut_src + ","
                for mut_dst in sorted(dst_muts):
                    dst_muts_txt += mut_dst + ","
                if src_muts_txt.endswith(","): src_muts_txt = src_muts_txt[:-1]
                if dst_muts_txt.endswith(","): dst_muts_txt = dst_muts_txt[:-1]
                
                text_line += src_muts_txt + "\t"
                text_line += dst_muts_txt + "\t"
                
                src_muts = set([])
                dst_muts = set([])
                src_muts_txt = ""
                dst_muts_txt = ""
                
                for (mut_src, mut_dst) in drug_res_conf_db.get(drug_name, {}).get(cluster_id, {}).get(position, set([])):
                    src_muts.add(mut_src)
                    dst_muts.add(mut_dst)
                
                if len(src_muts) > 1 and "X" in src_muts: src_muts.remove("X")
                
                for mut_src in src_muts:
                    src_muts_txt += mut_src + ","
                for mut_dst in sorted(dst_muts):
                    dst_muts_txt += mut_dst + ","
                if src_muts_txt.endswith(","): src_muts_txt = src_muts_txt[:-1]
                if dst_muts_txt.endswith(","): dst_muts_txt = dst_muts_txt[:-1]
                
                text_line += src_muts_txt + "\t"
                text_line += dst_muts_txt + "\t"
                
                if not drug_name in drug_res_all_db: text_line += "N"
                elif not cluster_id in drug_res_all_db[drug_name]: text_line += "N"
                elif not position in drug_res_all_db[drug_name][cluster_id]: text_line += "N"
                else: text_line += "Y"
                
                text_line += "\t"
                
                if not drug_name in drug_res_conf_db: text_line += "N"
                elif not cluster_id in drug_res_conf_db[drug_name]: text_line += "N"
                elif not position in drug_res_conf_db[drug_name][cluster_id]: text_line += "N"
                else: text_line += "Y"
                
                text_line += "\n"
                #text_lines.append(text_line)
                if not (gene_name, position) in used_gene_pos:
                    used_gene_pos.add((gene_name, position))
                    text_lines_best.append(text_line)
                
    
    output_fn = analysis_dir + "/tbdb_ranks_" + m_type + ".txt"    
    output_fh = open(output_fn, "w")
    for text_line in text_lines_best:
        output_fh.write(text_line)
    output_fh.close()
