import sys
import os
import shutil
import multiprocessing
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_progress_utils import GWADRIBProgress
from drsoft.utils.gwamar_ann_utils import readClusterGeneIDs
from drsoft.utils.gwamar_muts_utils import *
from drsoft.utils.gwamar_res_utils import compareProfiles
from drsoft.utils.gwamar_res_io_utils import readResistanceProfiles
from drsoft.utils.gwamar_muts_io_utils import readDrugResRel
from drsoft.utils.gwamar_strains_io_utils import readStrains

def loadDrugResMuts(input_fh, cluster_ids_rev={}):

    drug_targets = {}
    lines= input_fh.readlines();
    for line in lines:
        line = line.strip()

        tokens_hash = line.split("#")
        if len(tokens_hash) > 1: line = tokens_hash[0]
        tokens = line.split()
        
        if len(tokens) < 5: continue
        
        drug_name = tokens[0]
        if not drug_name in drug_targets:
            drug_targets[drug_name] = {}

        gene_id = tokens[2]
        position = int(tokens[3])
        ref_states = tokens[4]
        mut_states = tokens[5]
        
        cluster_id =  cluster_ids_rev.get(gene_id, gene_id)
       # print(line, cluster_id)

        if not cluster_id in drug_targets[drug_name]:
            drug_targets[drug_name][cluster_id] = {}
        if not position in drug_targets[drug_name][cluster_id]:
            drug_targets[drug_name][cluster_id][position] = set([])
        drug_targets[drug_name][cluster_id][position].add((ref_states, mut_states))
    return drug_targets

def loadDrugResRel(input_fh, cluster_ids_rev={}):

    drug_targets = {}
    lines= input_fh.readlines();
    for line in lines:
        line = line.strip()
        tokens_hash = line.split("#")
        if len(tokens_hash) > 1: line = tokens_hash[0]
        tokens = line.split()

        if len(tokens) < 5: continue
        
        drug_name = tokens[0]
        if not drug_name in drug_targets:
            drug_targets[drug_name] = {}
        gene_id = tokens[2]
        position = int(tokens[3])

        cluster_id =  cluster_ids_rev.get(gene_id, gene_id)

        start = position
        end = position
        
        if not cluster_id in drug_targets[drug_name]:
            drug_targets[drug_name][cluster_id] = [] 
        drug_targets[drug_name][cluster_id].append((start, end))
    return drug_targets

def readScoresMap(input_fh):
    scores_map = {}
    for line in input_fh.readlines():
        tokens = line.split()
        if len(tokens) < 2: continue
        bin_profile_id = tokens[0]
        score = float(tokens[1])
        scores_map[bin_profile_id] = score
    return scores_map

def mutationsIDmap(bin_profile_details):
    mutations_map = {}
    mut_key_id = 0

    for bin_profile_id in bin_profile_details:
        for (gene_id, cluster_id, gene_name, position, res_states_txt, mut_states_txt) in bin_profile_details.get(bin_profile_id, []):
            mut_key = gene_id +" " + cluster_id + " " + gene_name + " " + str(position) + " " + res_states_txt + " "+ mut_states_txt
            mutations_map[mut_key] = mut_key_id
            mut_key_id += 1
    return mutations_map

def readScoreMutations(bin_profile_details, scores_bin_map):
    n = len(scores_bin_map)
    if n == 0: return []
    mut_scores = {}

    for bin_profile_id in scores_bin_map:
        bin_profile_score = scores_bin_map[bin_profile_id]
        bin_profile = bin_profiles.get(bin_profile_id, None)
        if bin_profile == None: continue

        for (gene_id, cluster_id, gene_name, position, res_states_txt, mut_states_txt) in bin_profile_details.get(bin_profile_id, []):
            mut_key = gene_id +" " + cluster_id + " " + gene_name + " " + str(position) + " " + res_states_txt + " "+ mut_states_txt
#            mut_key += "\t" + db_all + "\t" + db_hc
            score = scores_bin_map.get(bin_profile_id, 0.0)
            mut_scores[(mut_key, bin_profile_id)] = score

    return mut_scores

def readGoldAssociations(bin_profile_details, drug_name):
    mut_assocs = {}
    for bin_profile_id in bin_profile_details:
        for (gene_id, cluster_id, gene_name, position, res_states_txt, mut_states_txt) in bin_profile_details.get(bin_profile_id, []):
            if drug_name in drug_res_all_db and cluster_id in drug_res_all_db[drug_name] and position in drug_res_all_db[drug_name][cluster_id]:
                db_all = "Y"
            elif drug_name in drug_res_all_db and gene_id in drug_res_all_db[drug_name] and position in drug_res_all_db[drug_name][gene_id]:
                db_all = "Y"
            elif drug_name in drug_res_conf_db and cluster_id in drug_res_conf_db[drug_name] and position in drug_res_conf_db[drug_name][cluster_id]:
                db_all = "Y"
            elif drug_name in drug_res_conf_db and gene_id in drug_res_conf_db[drug_name] and position in drug_res_conf_db[drug_name][gene_id]:
                db_all = "Y"
            else: db_all = "N"
            
            if drug_name in drug_res_conf_db and cluster_id in drug_res_conf_db[drug_name] and position in drug_res_conf_db[drug_name][cluster_id]:
                db_hc = "Y"
            elif drug_name in drug_res_conf_db and gene_id in drug_res_conf_db[drug_name] and position in drug_res_conf_db[drug_name][gene_id]:
                db_hc = "Y"
            else: db_hc = "N"
             #   db_gene_all = "Y"
            if drug_name in drug_res_all_db and (cluster_id in drug_res_all_db[drug_name] or gene_id in drug_res_all_db[drug_name]):
                db_gene_all = "Y"
            elif drug_name in drug_res_conf_db and (cluster_id in drug_res_conf_db[drug_name] or gene_id in drug_res_conf_db[drug_name]):
                db_gene_all = "Y"
            else: db_gene_all = "N"

            #if drug_name in drug_res_rel and (cluster_id in drug_res_rel[drug_name] or gene_id in drug_res_rel[drug_name]):
            #    db_gene_high = "Y"
            if drug_name in drug_res_conf_db and (cluster_id in drug_res_conf_db[drug_name] or gene_id in drug_res_conf_db[drug_name]):
                db_gene_high = "Y"
            else: db_gene_high = "N"

            mut_key = " ".join([gene_id, cluster_id, gene_name, str(position), res_states_txt, mut_states_txt])

            mut_assocs[(drug_name, mut_key)] = "\t".join([db_all, db_hc, db_gene_all, db_gene_high])
    
    return mut_assocs

def saveAssociations(params):

    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()

    dataset = parameters["D"]
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    ranks_dir = parameters["RESULTS_RANKS_DIR"]
    analysis_dir = parameters["RESULTS_ANALYSIS_DIR"]
    revord_scores = parameters["REVORDER"].split(",")

    score, gold_assocs, mutations_map, drug_names_map, drug_res_rel = params
    
    statistics = {}
    associations = {}

    drug_names_map_rev = {}
    for drug_name in drug_names_map:
        drug_name_id = drug_names_map[drug_name]
        drug_names_map_rev[drug_name_id] = drug_name

    mutations_map_rev = {}
    for mutkey in mutations_map:
        mutkey_id = mutations_map[mutkey]
        mutations_map_rev[mutkey_id] = mutkey

    if score in revord_scores or score.count("+") > 0 or score.startswith("pv-"): revord = 1
    else: revord = -1
                     
    for drug_res_profile in drug_res_profiles:
        dr_profile = drug_res_profile.full_profile
        drug_name = drug_res_profile.drug_name

        bin_profiles_res_rel = bin_profile_details

        input_fn = scores_dir + "/" + drug_name  + "/" + score + ".txt"
        if not os.path.exists(input_fn) or os.path.getsize(input_fn) < 10:
            print("NOT EXIST: ", input_fn)
            output_fn = analysis_dir + "/assocs_latex_all/" + score + ".txt"
            if os.path.exists(output_fn):
                os.remove(output_fn)
            return score
        else:
            input_fh = open(input_fn)
            bin_scores_map = readScoresMap(input_fh)
            input_fh.close()
            

        mut_scores = readScoreMutations(bin_profiles_res_rel, bin_scores_map)
        drug_name_id = drug_names_map[drug_name]
        for (mutkey, bin_profile_id) in mut_scores:
            mutkey_id = mutations_map[mutkey]
            bin_profile = bin_profiles[bin_profile_id]

            associations[(drug_name_id, mutkey_id)] = mut_scores[(mutkey, bin_profile_id)]

            s0, s1, i0, i1, r0, r1 = compareProfiles(bin_profile, dr_profile)  
            statistics[(drug_name_id, mutkey_id)] = str(s0)+"\t"+str(s1)+"\t"+str(i0)+"\t"+str(i1)+"\t"+str(r0)+"\t"+str(r1)

    associations_sorted = sorted(associations, key=lambda assoc_key: revord*associations[assoc_key])
            
    output_fn = analysis_dir + "/assocs_latex_all/" + score + ".txt"    
    output_fh = open(output_fn, "w")
    for (drug_name_id, mutkey_id) in associations_sorted:
        score_val = associations[(drug_name_id, mutkey_id)]
        stats_tmp = statistics[(drug_name_id, mutkey_id)]
        if not (drug_name_id, mutkey_id) in gold_assocs:
            continue
        gold_assoc = gold_assocs[(drug_name_id, mutkey_id)]
        #if gold_assoc.endswith("N"):
        #    continue
        drug_name = drug_names_map_rev[drug_name_id]
        mutkey = mutations_map_rev[mutkey_id]
  #      tl = gold_assoc + "\t" + str(score_val) + "\t"+ str(drug_name) + "\t" + str(mutkey) + "\t" + stats_tmp
        tokens = mutkey.split()
        tokens_gold = gold_assoc.split()
        db_all,db_high = tokens_gold[0:2]
        gene_id = tokens[0]
        gene_name, position, ref_mut, mut_mut = tokens[2:6]
        tl = " ".join([drug_name, gene_id, gene_name, ref_mut+position+mut_mut.replace(",","/"),db_all,db_high])
        output_fh.write(tl + "\n")
    output_fh.close()
    return score

if __name__ == '__main__':
    print("tb4_save_scored_lists_latex")
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    dataset = parameters["D"]
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    ranks_dir = parameters["RESULTS_RANKS_DIR"]
    analysis_dir = parameters["RESULTS_ANALYSIS_DIR"]
    revord_scores = parameters["REVORDER"].split(",")
      
    WORKERS = min(int(parameters["W"]), multiprocessing.cpu_count())
    

    ensure_dir(analysis_dir)
    ensure_dir(analysis_dir + "/assocs_latex_all/")
    
    input_fh = open(input_dir + "/strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    m_type = parameters["MT"]
    
    input_fh = open(results_dir + "/res_profiles.txt")
    drug_res_profiles = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    input_fh = open(input_dir + 'bin_profiles_' + m_type + '_det.txt')
    bin_profile_details = readBinProfileDetails(input_fh, mut_type=m_type, out_format="tuple")
    input_fh.close()

    input_fh = open(input_dir + 'bin_profiles_' + m_type + '.txt')
    bin_profiles = readBinProfiles(input_fh, strains.count(), gen_bin_profiles=True)
    input_fh.close()
    
    if os.path.exists(input_dir + "/cluster_gene_ref_ids.txt"):
        cluster_fh = open(input_dir + "/cluster_gene_ref_ids.txt")
        cluster_ids, cluster_ids_rev = readClusterGeneIDs(cluster_fh, gene_ref_ids=True)
        cluster_fh.close()
    else:
        cluster_ids = {} 
        cluster_ids_rev = {}

    if m_type == "P":
        input_fh = open(input_dir + "/drug_res_"+m_type+"_G_M.txt")
        drug_res_rel = readDrugResRel(input_fh, cluster_ids_rev)
        input_fh.close()
    else:
        drug_res_rel = {}
        
    input_fh = open(input_dir + "/drug_res_"+m_type+"_H.txt")
    drug_res_conf_db = loadDrugResMuts(input_fh, cluster_ids_rev)
    input_fh.close()
    
    input_fh = open(input_dir + "/drug_res_"+m_type+"_A.txt")
    drug_res_all_db = loadDrugResMuts(input_fh, cluster_ids_rev)
    input_fh.close()


    score_params_union = set([])
    
    drug_names_map = {}
    drug_name_id = 0
    for drug_res_profile in drug_res_profiles:
        drug_name = drug_res_profile.drug_name

        for score_fn in os.listdir(scores_dir + "/" + drug_name):
            input_fn = scores_dir + "/" + drug_name  + "/" + score_fn
            if os.path.exists(input_fn): 
                score_params_union.add(score_fn[:-4])
                if not drug_name in drug_names_map:
                    drug_names_map[drug_name] = drug_name_id
                    drug_name_id += 1

    score_params_union = score_params_union & set(["ws","mi","or","lh","r-fcctw2"])

    print("drug_res_all_db", len(drug_res_all_db))
    print("drug_res_conf_db", len(drug_res_conf_db))
    print("drug_res_rel", len(drug_res_rel))

    
    mutations_map = mutationsIDmap(bin_profile_details)

    gold_assocs = {}

    for drug_res_profile in drug_res_profiles:
        dr_profile = drug_res_profile.full_profile
        drug_name = drug_res_profile.drug_name
        gold_assoscs_tmp = readGoldAssociations(bin_profile_details,drug_name)
        for (drug_name, mutkey) in gold_assoscs_tmp:
            drug_name_id = drug_names_map[drug_name]
            mutkey_id = mutations_map[mutkey]
            gold_assocs[(drug_name_id, mutkey_id)] = gold_assoscs_tmp[(drug_name, mutkey)]


    TASKS = []
    for score in list(score_params_union):
        TASKS.append((score, gold_assocs, mutations_map, drug_names_map, drug_res_rel))

    progress = GWADRIBProgress("Associations")
    progress.setJobsCount(len(TASKS))
    if WORKERS > 1:
        pool = Pool(processes=WORKERS)
        for r in pool.imap(saveAssociations, TASKS):
            progress.update(str(r))
    else:
        for T in TASKS:
            r = saveAssociations(T)
            progress.update(str(r))

