import os
import sys
import time
import random
import copy
import shutil
from collections import deque

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.utils.gwamar_tree_io_utils import *
from drsoft.structs.res_tree import ResTreeNode

def generateBinaryTree(n):
    bin_tree = ResTreeNode(1, None)
    
    queue = deque()
    queue.append(bin_tree)
    i = 1
    index = 0
    
    while i < n:
        node = queue.popleft()
        
        left = ResTreeNode(index, node)
        right = ResTreeNode(index+1, node)
        
        node.children = [left, right]
        queue.append(left)
        queue.append(right)
        i += 1
        index += 2
    return bin_tree

def binarizeTreeRandom(tree):
    queue = deque()
    queue.append(tree.root)
    
    while queue:
        node = queue.popleft()
        if len(node.children) == 0:
            pass
        elif len(node.children) == 2:
            for child_node in node.children:
                queue.append(child_node)
        elif len(node.children) == 1:
            print("something wrong,1 ")
        else:
            n = len(node.children)
            tree_tmp = generateBinaryTree(n)

            perm = range(n)
            random.shuffle(perm)
 
            node_children = copy.deepcopy(node.children)
            node.children = tree_tmp.children
            
            leaf_nodes = tree_tmp.getLeaves()

            for new_child in node.children:
                new_child.parent = node
            
            for i in range(n):
                index = perm[i]
                
                child_node = node_children[i]
                leaf_node = leaf_nodes[index]
                        
                leaf_node.children = child_node.children
                leaf_node.node_id = child_node.node_id
                
                #print(i, index, child_node.node_id, leaf_node.node)
                
                for ch_node in leaf_node.children:
                    ch_node.parent = leaf_node
                    
                queue.append(leaf_node)
            
                            
            #print("before", len(node.children))
            
            #print("after", len(node.children))
            

            
    return tree

def randomBinaryTrees(input_fh, output_fh, strains, k):
    trees = readPhyloTrees(input_fh, strains_list)
    tree = list(trees)[0]
    flat_tree = tree.flatten(0.00001)
    
    for i in range(k):
        flat_tree_tmp = copy.deepcopy(flat_tree)
        
        random_bin_tree = binarizeTreeRandom(flat_tree_tmp)
        random_tree_txt = random_bin_tree.toString(lengths=False, strains=strains)
        
        output_fh.write(random_tree_txt+ ";\n")
    return None

if __name__ == '__main__':
    print("p4_gen_binary_trees: binarizes the given tree (randomly).")

    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    WORKERS = 1#min(int(parameters["W"]), multiprocessing.cpu_count())
        
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
        
    input_fh = open(input_dir + "strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    strains_list = strains.allStrains()

    input_fn = results_dir + "tree.txt"
    input_fh = open(input_fn)
    tree = readPhyloTrees(input_fh, strains_list)
    input_fh.close()
  
    results_dir = output_dir = parameters["DATASET_DIR"] + "/"+parameters["EP"]+"G/"
    output_fn = results_dir + "tree_bin.txt"
    input_bin_fn = input_dir + "tree_bin.txt"
    if os.path.exists(input_bin_fn):
        shutil.copy2(input_bin_fn, output_fn)
    else:
        input_fn = input_dir + "tree.txt"
        input_fh = open(input_fn)
        output_fh = open(output_fn, "w")
        randomBinaryTrees(input_fh, output_fh, strains, 5)
        output_fh.close();
        input_fh.close()

    results_dir = output_dir = parameters["DATASET_DIR"] + "/"+parameters["EP"]+"P/"
    output_fn = results_dir + "tree_bin.txt"
    input_bin_fn = input_dir + "tree_bin.txt"
    if os.path.exists(input_bin_fn):
        shutil.copy2(input_bin_fn, output_fn)
    else:
        input_fn = input_dir + "tree.txt"
        input_fh = open(input_fn)
        output_fh = open(output_fn, "w")
        randomBinaryTrees(input_fh, output_fh, strains, 5)
        output_fh.close();
        input_fh.close()
    print("p4_gen_binary_trees: Finished.")
