import sys
import os
import copy
import multiprocessing
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_ann_utils import *
from drsoft.utils.gwamar_muts_utils import *
from drsoft.utils.gwamar_res_utils import *
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_strains_io_utils import *
from drsoft.utils.gwamar_muts_io_utils import *
from drsoft.utils.gwamar_res_io_utils import *
from drsoft.utils.gwamar_tree_io_utils import *
from drsoft.utils.gwamar_progress_utils import *
from drsoft.utils.gwamar_scoring_flat_utils import *
from drsoft.utils.gwamar_scoring_tree_utils import *
from drsoft.utils.gwamar_scoring import *
from drsoft.utils.gwamar_utils import ensure_dir

def scoreDrug(params):
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    drug_name = params["DRUG_NAME"]
    score_full = params["SCORE"]

    bin_profile_scores, rev_sorting = computeScores(score_full, drug_name, parameters)

    if bin_profile_scores == None:

        return drug_name, score_full
    
    text_lines = []
    bin_profile_scores_sorted = sorted(bin_profile_scores, key=lambda bin_profile_id:rev_sorting*bin_profile_scores[bin_profile_id])

    for bin_profile_id in bin_profile_scores_sorted:
        bin_profile_score = bin_profile_scores[bin_profile_id]
        text_line = str(bin_profile_id) + "\t" + str(bin_profile_score) + "\n"
        text_lines.append(text_line)

    ensure_dir(parameters["RESULTS_SCORES_DIR"] + drug_name + "/")
    output_fh = open(parameters["RESULTS_SCORES_DIR"] + drug_name + "/" + score_full + ".txt", "w") 
    for text_line in text_lines:
        output_fh.write(text_line)
    output_fh.close()
    
    return drug_name, score_full
    
if __name__ == '__main__':
    print("sc1_compute_std_scores.py. Computation of scores...")

    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    WORKERS = min(int(parameters["W"]), multiprocessing.cpu_count())
    
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    
    ensure_dir(results_dir)
    ensure_dir(scores_dir)
    ensure_dir(parameters["TGH_DIR"])
    
    print("Scores to compute: " + parameters["SCORES"])
    
    score_params = parameters["SCORES"].split(",")
    
    TASKS = []
    TASK = {}
    
    input_fn = results_dir + "/res_profiles.txt"
    input_fh = open(input_fn)
    drug_names = readDrugNames(input_fh)
    input_fh.close()

    print("Drug resistance profiles: " + ' '.join(drug_names))
    
    for drug_name in drug_names:
        ensure_dir(scores_dir + drug_name)
        for score_param in score_params:
            if parameters["OW"] == "N" and os.path.exists(parameters["RESULTS_SCORES_DIR"] + drug_name + "/" + score_param + ".txt"):
                continue
            TASK["DRUG_NAME"] = drug_name
            TASK["SCORE"] = score_param
            TASKS.append(TASK.copy())

    print("Total number of jobs: " + str(len(TASKS)))

    progress = GWAMARProgress("Scores computed")
    progress.setJobsCount(len(TASKS))
    if WORKERS > 1:
        pool = Pool(processes=WORKERS)
        for r in pool.imap(scoreDrug, TASKS):
            progress.update(str(r))
    else:
        for T in TASKS:
            r = scoreDrug(T)
            progress.update(str(r))

    print("sc1_compute_std_scores.py. Finished")


