import sys
import os
import copy
from multiprocessing import Pool
import multiprocessing

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_progress_utils import GWAMARProgress
from drsoft.utils.gwamar_res_io_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains

def normalizeBinScores(bin_profiles_sorted):
    n = len(bin_profiles_sorted)
    norm_bin_profiles_sorted = []
    first_score = bin_profiles_sorted[0][1]
    last_score = bin_profiles_sorted[-1][1]
    for i in range(n):
        bin_profile_id, bin_profile_score = bin_profiles_sorted[i]
        if last_score == first_score: norm_bin_profile_score = 0.0
        else: norm_bin_profile_score = float(last_score - bin_profile_score)/float(last_score - first_score)
        norm_bin_profiles_sorted.append((bin_profile_id, norm_bin_profile_score))
    return norm_bin_profiles_sorted

def scoreDrug(params):
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()

    drug_name = params["DRUG_NAME"]
    score = params["SCORE"]

    input_dir = parameters["DATASET_INPUT_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    ranks_dir = parameters["RESULTS_RANKS_DIR"]
    results_dir = parameters["RESULTS_DIR"]


    rev_order = False
    if score.startswith("pv-") or score.count("+")>0:
        rev_order = True

    input_fh = open(input_dir + "/strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    input_fh = open(results_dir + "/res_profiles.txt")
    res_profiles_list = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    input_fh = open(results_dir + '/bin_profiles_' + parameters["MT"] + '_count.txt')
    bin_profile_count = readBinProfileCounts(input_fh)
    input_fh.close()
    
    res_profiles = {}
    
    for res_profile in res_profiles_list:
        res_profiles[res_profile.drug_name] = res_profile
    drug_res_profile = res_profiles[drug_name]
    input_fn = scores_dir + drug_name + "/" + score + ".txt"
    if not os.path.exists(input_fn) or os.path.getsize(input_fn) < 10:
        print("NOT EXIST: ", input_fn)
        return drug_name, score

    ensure_dir(ranks_dir + drug_name)
            
    input_fh = open(input_fn)
    bin_profiles_sorted = readBinProfilesSorted(input_fh, rev_order)
    input_fh.close()
    
    norm_bin_profiles_sorted = normalizeBinScores(bin_profiles_sorted)        

    output_fh = open(scores_dir + drug_name + "/n-" + score + ".txt", "w") 
    for (bin_profile_id, bin_profile_score) in norm_bin_profiles_sorted:
        output_fh.write(str(bin_profile_id) + "\t" + str(bin_profile_score) + "\n")
    output_fh.close()
    
    return drug_name, score
    
if __name__ == '__main__':
    print("sc5_compute_norm_scores.py. Normalization of scores.")
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    WORKERS = min(int(parameters["W"]), multiprocessing.cpu_count())
    
    
    TASKS = []
    TASK = {}
    
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    
    input_fn = results_dir + "/res_profiles.txt"
    input_fh = open(input_fn)
    drug_names = readDrugNames(input_fh)
    input_fh.close()
    
    for drug_name in drug_names:
        for score_fn in os.listdir(scores_dir + drug_name):
            score = score_fn[:-4]
            if score.startswith("pv-") or score.startswith("n-"):
                continue
         #   output_fn = scores_dir + drug_name + "/n-" + score_fn
         #   if os.path.exists(output_fn): continue
            
            TASK["DRUG_NAME"] = drug_name
            TASK["SCORE"] = score

            TASKS.append(TASK.copy())

    progress = GWAMARProgress("Scores normalized")
    progress.setJobsCount(len(TASKS))
    
    if WORKERS > 1:
        pool = Pool(processes=WORKERS)
        for r in pool.imap(scoreDrug, TASKS):
            progress.update(str(r))
    else:
        for T in TASKS:
            r = scoreDrug(T)
            progress.update(str(r))
    print("sc5_compute_norm_scores.py. Finished.")
    