def profileSubset(full_profile, subset):
    subset_profile = []
    for i in sorted(subset):
        subset_profile.append(full_profile[i])
    return subset_profile

def convertFullProfile(full_profile_tmp, strain_ids, strains_map_rev):
    if strains_map_rev == None or strain_ids == None:
        return full_profile_tmp

    n = len(full_profile_tmp)
    m = len(strains_map_rev)

    full_profile = [""]*m

    for i in range(n):
        val = full_profile_tmp[i]
        strain_name = strain_ids[i]
        if strain_name in strains_map_rev:
            j = strains_map_rev[strain_name]
            full_profile[j] = val
        else: continue

    return full_profile

def decompressProfile(m_desc, n, strain_index_map=None):
    if m_desc.count(" ") > 0:    
        sel_state, desc = m_desc.split() 
        full_profile = [sel_state]*n
        desc_tokens = desc.split(";")
        for desc_token in desc_tokens:
            curr_state, strains_desc = desc_token.split(":")
            strains_tokens = strains_desc.split(",")
            for strain_token in strains_tokens:
                strain_token_dets = strain_token.split("_")
                if len(strain_token_dets) == 1:
                    strain_index = int(strain_token_dets[0])
                    if strain_index_map == None:
                        full_profile[strain_index] = curr_state
                    else:
                        full_profile[strain_index_map[strain_index]] = curr_state
                elif len(strain_token_dets) == 2:
                    for strain_index in range(int(strain_token_dets[0]), int(strain_token_dets[1])+1, 1):
                        if strain_index_map == None:
                            full_profile[strain_index] = curr_state
                        else:
                            full_profile[strain_index_map[strain_index]] = curr_state
                else:
                    print("error", strain_token_dets)
    else:
        full_profile = []
        for i in range(n): full_profile.append(m_desc[i])
    return full_profile

def compressProfile(full_profile):
    n = len(full_profile)
            
    state_strains = {}
    for strain_index in range(n):
        state = full_profile[strain_index]
        if not state in state_strains:
            state_strains[state] = []
        state_strains[state].append(strain_index)

    sel_state = sorted(state_strains, key=lambda state:-len(state_strains[state]))[0]

    comp_profile = sel_state + " "
    
    for state in state_strains:
        if state == sel_state: continue
        comp_profile += state + ":"
        
        k = len(state_strains[state])
        i = 0
        while i < k:
            strain_index = state_strains[state][i]
            comp_profile += str(strain_index)
            j = 1
            while i+j<k and state_strains[state][i+j] == strain_index+j:
                j = j + 1
            i = i + j
            
            if j == 1: comp_profile += ","
            elif j == 2: comp_profile += "," + str(state_strains[state][i-1]) + ","
            else: comp_profile += "_" + str(state_strains[state][i-1]) + ","
                       
        if comp_profile.endswith(","): comp_profile = comp_profile[:-1] + ";"

    if comp_profile.endswith(";"): comp_profile = comp_profile[:-1]
    return comp_profile

def subsets(my_set):
    result = [[]]
    for x in my_set:
        result = result + [y + [x] for y in result]
    return result


def calcMutStatesMutation(m_profile, ref_state, indel='0'):
    if indel == '0': non_mut_states = ["?", "-"]
    else: non_mut_states = ["?"]
    mut_states = []
    if m_profile[1] == " ":
        tokens = m_profile.split()
        state_pos = {}
        state = tokens[0]
        if state != ref_state and not state in non_mut_states:
            last_pos = 0
            tokens2 = tokens[1].split(";")

            for state_token in tokens2:
                t0, t1 = state_token.split(":")
                last_pos = max(last_pos, int(t1.split("_")[-1].split(",")[-1]))

            ref_positions = set(range(last_pos + 2))

            for state_token in tokens2:
                t0, t1 = state_token.split(":")
                for t1x in t1.split(","):
                    if t1x.count("_") == 0:
                        ref_positions.remove(int(t1x))
                    else:
                        x_str, y_str = t1x.split("_")
                        x = int(x_str)
                        y = int(y_str)
                        for p in range(x, y+1, 1):
                            ref_positions.remove(p)
            state_pos[state] = min(ref_positions)

        tokens2 = tokens[1].split(";")
        for state_token in tokens2:

            t0, t1 = state_token.split(":")
            if t0 in non_mut_states or t0 == ref_state:
                continue
            state = t0
            state_pos[state] = int(t1.split("_")[0].split(",")[0])

        mut_states = sorted(state_pos, key=lambda state: state_pos[state])
    else:
        n = len(m_profile)
        for i in range(n):
            state = m_profile[i]
            if state != ref_state and not state in non_mut_states:
                if not state in mut_states:
                    mut_states.append(state)
    
    mut_states_txt = ",".join(mut_states)
    
    if ref_state in mut_states_txt:
        print("zle",ref_state, mut_states_txt, m_profile)
    return mut_states_txt

def calcRefStateMutation(full_profile, strain_subset=None):
    non_ref_states = ["?","*",'-']
    state_counts = {};
    n = len(full_profile)
    if strain_subset == None: strain_subset = range(n)
    for strain_index in strain_subset:
        state = full_profile[strain_index]
        if state in non_ref_states:
            continue
        if not state in state_counts:
            state_counts[state] = 0
        state_counts[state] += 1;
    
    if len(state_counts)== 0:
        strain_subset = range(n)
        for strain_index in strain_subset:
            state = full_profile[strain_index]
            if state in non_ref_states:
                continue
            if not state in state_counts:
                state_counts[state] = 0
            state_counts[state] += 1;
            
    if len(state_counts)== 0:
        return None
        
    state_counts_sorted = sorted(state_counts, key=lambda state: -state_counts[state])
    max_state = state_counts_sorted[0]
    
    return max_state

def calcBinProfileMutation(full_profile, ref_state, gen_bin_profile=False, indel='0'):
    n = len(full_profile)
    bin_profile_list = []
    if not gen_bin_profile:
        for i in range(n):
            state = full_profile[i]
            if state == "?": bin_profile_list.append("?")
            elif state == ref_state or state == '-': bin_profile_list.append("0")
            else: bin_profile_list.append("1")
    else:
        current_k = 1
        k_map = {}
        for i in range(n):
            state = full_profile[i]
            if state == "?": bin_profile_list.append("?")
            elif state == ref_state or state == '-': bin_profile_list.append("0")
            elif state in k_map: bin_profile_list.append(str(k_map[state]))
            else:
                if current_k >= 10:
                    print(current_k, full_profile)
                    print(k_map)
                    exit()
                k_map[state] = current_k
                bin_profile_list.append(str(current_k))
                current_k += 1
                
    return bin_profile_list

def getRefMutStates(full_profile, bin_profile):
    ref_states = set([])
    mut_states = set([])
    n = len(full_profile)
    for i in range(n):
        bin_state = bin_profile[i]
        state = full_profile[i]
        if state == "?": continue
        if bin_state == '1': mut_states.add(state)
        elif bin_state == '0': ref_states.add(state)
    return ref_states, mut_states
    
def calcRefPosition(position, start, end, strand):
    left_bound = min(start, end)
    right_bound = max(start, end)
    if strand == "+":
        if position <= 0: ref_position = left_bound + position
        else: ref_position = left_bound + (position-1)*3 + 1
    else:
        if position <= 0: ref_position = right_bound - position + 1
        else: ref_position = right_bound - (position-1)*3 - 1 
    
    return ref_position

def reverseBinProfileIDs(gene_profiles, muts=True):
    bin_profile_ids = {}
    for gene_profile in gene_profiles.values():
        for mutation in gene_profile.mutations.values():
            for bin_profile_id in mutation.bin_profiles:
                if not bin_profile_id in bin_profile_ids:
                    bin_profile_ids[bin_profile_id] = set([])
                bin_profile_ids[bin_profile_id].add(mutation)
    return bin_profile_ids
