from drsoft.structs.resistance_profile import ResistanceProfile
from drsoft.utils.gwamar_muts_utils import compressProfile, decompressProfile,convertFullProfile
from drsoft.utils.gwamar_muts_io_utils import writeMutationsHeader
from drsoft.utils.gwamar_strains_io_utils import strainAureusNames,\
    strainsMapRev

def saveResistanceProfiles(output_fh, res_profiles_list, strains_list, compress=False, header=True, sel_profiles=None):
    text_lines = []
    if compress == False:
        for drp in res_profiles_list:
            drug_name = drp.drug_name

            if sel_profiles != None and not drug_name in sel_profiles:
                continue
            text_line = drug_name+"\t"
            for i in drp.getResistant():
                if i in drp.pred_strains:
                    text_line += "*"
                text_line += strains_list[i]+";"
            if text_line.endswith(";"): text_line = text_line[:-1]
            text_line += "\t"
            for i in drp.getSusceptible():
                if i in drp.pred_strains:
                    text_line += "*"
                text_line += strains_list[i]+";"
            if text_line.endswith(";"): text_line = text_line[:-1]
            text_line += "\t"
            for i in drp.getInterresistant():
                if i in drp.pred_strains:
                    text_line += "*"
                text_line += strains_list[i]+";"
            
            if text_line.endswith(";"): text_line = text_line[:-1]
            text_line += "\n"
            text_lines.append(text_line)
    else:
        if header:
            writeMutationsHeader(output_fh, strains_subset=strains_list, only_strains=True, sep=" ")
        for drp in res_profiles_list:
            drug_name = drp.drug_name
            if sel_profiles != None and not drug_name in sel_profiles:
                continue
            cmp_profile = compressProfile(drp.full_profile)
            text_line = drug_name + "\t" + cmp_profile + "\n"
            text_lines.append(text_line)
        
    for text_line in text_lines:
        output_fh.write(text_line)
    return None

def readResistanceProfiles(input_fh, strains_list = None, load_int = False, as_map = False):
    res_profiles_list = []
    
    lines = input_fh.readlines()
    line0 = lines[0].strip()
    strain_ids_tmp = line0.split()
    strain_ids = []
    for strain_id_tmp in strain_ids_tmp:
        strain_ids.append(strainAureusNames(strain_id_tmp))
    n = len(strain_ids)

    if strains_list == None:
        strains_map_rev = None
    else:
        strains_map_rev = strainsMapRev(strains_list)
           
    # strains_list = line0.split()
    # n = len(strains_list)
    # strains_map_rev = strainsMapRev(strains_list)

    for line in lines[1:]:
        line = line.strip()
        if line.startswith("#"): continue
        tokens = line.split("\t")
        drug_name = tokens[0]
        if len(tokens) < 2: continue
        cmp_profile = tokens[1]

        full_profile_tmp = decompressProfile(cmp_profile, n)
        full_profile = convertFullProfile(full_profile_tmp, strain_ids, strains_map_rev)
       # print(len(full_profile_tmp), len(full_profile))
        res_profile = ResistanceProfile(drug_name, len(full_profile))
        res_profile.full_profile = full_profile
        res_profile.computeResSets()
        res_profiles_list.append(res_profile)
    # else:
    #     n = len(strains_list)
    #     strains_map_rev = strainsMapRev(strains_list)
    #     for line in lines:
    #         line = line.strip()
    #         if line.startswith("#"): continue
    #         if len(line) < 2: continue
    
    #         tokens = line.split("\t")
    #         if len(tokens) < 3: continue
            

    #         drug_name = tokens[0]
    #         res_strains_tmp = tokens[1].split(";")
    #         sc_strains_tmp = tokens[2].split(";")
            
    #         if len(tokens) == 4 and load_int: int_strain_tmp = tokens[3].split(";")
    #         else: int_strain_tmp = []
            
    #         res_strains = set([])
    #         sc_strains = set([])
    #         int_strains = set([])
    #         pred_strains = set([])
            
    #         for strain_id in res_strains_tmp:
    #             if len(strain_id) < 1: continue
    #             if strain_id.startswith("*"): strain_id = strain_id[1:]
    #             strain_id = strainAureusNames(strain_id)

    #             res_strains.add(strain_id)
    #         for strain_id in sc_strains_tmp:
    #             if len(strain_id) < 1: continue
    #             if strain_id.startswith("*"): strain_id = strain_id[1:]
    #             strain_id = strainAureusNames(strain_id)
                
    #             sc_strains.add(strain_id)
    #         for strain_id in int_strain_tmp:
    #             if len(strain_id) < 1: continue
    #             if strain_id.startswith("*"): strain_id = strain_id[1:]
    #             strain_id = strainAureusNames(strain_id)
                
    #             int_strains.add(strain_id)
                
    #         res_profile = ResistanceProfile(drug_name, n)
            
    
    #         for strain_id in strains_list:
    #             strain_index = strains_map_rev[strain_id]
    #             if strain_id in sc_strains:
    #                 res_profile.changeState(strain_index, "S")
    #             elif strain_id in res_strains:
    #                 res_profile.changeState(strain_index, "R")
    #             elif strain_id in int_strains:
    #                 res_profile.changeState(strain_index, "I")
    #             else:
    #                 res_profile.changeState(strain_index, "?")
            
    #         print(''.join(res_profile.full_profile))
    #         res_profile.computeResSets()
            
    #         res_profile.drug_name = drug_name
    #         res_profile.pred_strains = pred_strains
    #         res_profiles_list.append(res_profile)
    if as_map == True:
        res_profiles_map = {}
        
        for res_porfile in res_profiles_list:
            res_profiles_map[res_porfile.drug_name] = res_porfile
            
        return res_profiles_map
    
    return res_profiles_list


def readDrugNames(input_fh):
    drug_names = set([])
    lines= input_fh.readlines();
    
    for line in lines:
        if line.startswith("#"): continue
        line = line.strip()
        tokens = line.split("\t")
        if len(tokens) < 2: continue
        drug_name = tokens[0]
        drug_names.add(drug_name)
    return drug_names
