import sys
import os
import copy
from multiprocessing import Pool
import multiprocessing

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_utils import *
from drsoft.utils.gwamar_muts_io_utils import *
from drsoft.utils.gwamar_strains_io_utils import *
from drsoft.structs.resistance_profile import ResistanceProfile

def ensure_rem(filename):
    if os.path.exists(filename):
        os.remove(filename)

def computeTreeDNAML(input_fn, outfile_fn, outtree_fn, phylip_path, sufix = ""):
    abs_phylip_path = phylip_path
    if not os.path.exists(abs_phylip_path):
        print("Cannot find PHYLIP at this path: " + abs_phylip_path)

    os.chdir(phylip_path)   
    ensure_rem(outfile_fn)
    ensure_rem(outtree_fn)
    tmp_fn = "tmpfile" + sufix
    tmp_fh = open(tmp_fn, "w")
    tmp_fh.write(input_fn + "\n")
    tmp_fh.write("F" + "\n")
    tmp_fh.write(outfile_fn + "\n")
    tmp_fh.write("Y" + "\n")
    tmp_fh.write("F" + "\n")
    tmp_fh.write(outtree_fn + "\n")
    tmp_fh.close()
    if platform.system().count("WIN") > 0:
        os.system(abs_phylip_path + "dnamlk < " + tmp_fn)
    else:
        os.system(abs_phylip_path + "/dnamlk < " + tmp_fn)

def computeTreePROML(input_fn, outfile_fn, outtree_fn, sufix = ""):
    if platform.system().count("WIN") > 0:
        phylip_path = "C:/Users/workshop/Dropbox/camber/phylip/exe"
        if platform.system().upper().count("CYGWIN") > 0:
            input_fn.replace("/cygdrive/c/", "C:/")
            outfile_fn.replace("/cygdrive/c/", "C:/")
            outtree_fn.replace("/cygdrive/c/", "C:/")
            abs_phylip_path = phylip_path        
    else:    
        phylip_path = "/home/misias/phylip/exe/"
        abs_phylip_path = os.path.abspath(phylip_path)
    os.chdir(phylip_path)
    ensure_rem(outfile_fn)
    ensure_rem(outtree_fn) 
    print(os.path.curdir, abs_phylip_path)
    print(input_fn)
    print(outfile_fn)
    print(outtree_fn)
    tmp_fn = "tmpfile" + sufix
    tmp_fh = open(tmp_fn, "w")
    tmp_fh.write(input_fn + "\n")
    tmp_fh.write("F" + "\n")
    tmp_fh.write(outfile_fn + "\n")
    tmp_fh.write("Y" + "\n")
    tmp_fh.write("F" + "\n")
    tmp_fh.write(outtree_fn + "\n")
    tmp_fh.close()
    if platform.system().count("dows") > 0:
        os.system("promlk < " + tmp_fn)
    else:
        os.system(abs_phylip_path + "/promlk < " + tmp_fn)

    return None

if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    input_download_dir = parameters["PREBROAD_DOWNLOAD_DIR"]
    phylo_dir = parameters["PREBROAD_TREE_DIR"]
    input_dir = parameters["PREBROAD_OUTPUT_DIR"]
    m_type = parameters["PREBROAD_SUB"]
    phylip_path = parameters["PHYLIP_PATH"]
    
    input_fh = open(input_dir + "/strains.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    strains_ord = sorted(strains.allStrains())
    
    input_fn = phylo_dir + 'alignment_cluster_'+m_type+'.txt'
    outfile_fn = phylo_dir + 'phylofile_'+m_type+'.txt'
    outtree_fn = phylo_dir + 'phylotree_'+m_type+'.txt'
    
    print(input_fn, outtree_fn)
    
    computeTreeDNAML(input_fn, outfile_fn, outtree_fn, phylip_path, sufix = "_")
    print("t2_compute_subset_tree.py. Finished.")
    