#install.packages(
require(graphics); require(grDevices);

#data_fn = paste(species_id, "_out/r_subset/", drug_name, ".txt", sep = "")

args <- commandArgs(trailingOnly = TRUE)

scoreFullName <- function(score) {
	score_full = score
	if (score == "mi") {
		score_full = "mutual information";
	} else if (score == "or") {
		score_full = "odds ratio";
	} else if (score == "lh") {
		score_full = "hypergeometric score";
	} else if (score == "r-tgh") {
		score_full = "tree-generalized hypergeomtric score"
	} else if (score == "ws"){
		score_full = "weighted support"
	} else if(score == "ws+lh+r-tgh+mi+or") {
		score_full = "combined"
	} else if(score == "ts") {
		score_full = "tree score"
	}
	return(score_full);
}

if (length(args) >= 2) {
	mut_type = args[2]
} else {
	mut_type = "P"
}

if (length(args) >= 3) {
	subset = args[3]
} else {
	subset = "sel3"
}

if (length(args) >= 4) {
	prefix = args[4]
} else {
	if (Sys.info()["sysname"] == "Linux") {
		prefix = "/home/misias/Dropbox/gwamar/"
	} else {
		prefix = "D:/Dropbox/Dropbox/gwamar/"
	}
}


outdir = paste(prefix,"datasets/paper/", sep="")
dir.create(outdir, showWarnings = FALSE)

output_fn = paste(outdir, "/pr_", subset, ".pdf", sep="")
pdf(output_fn, width=18,height=10)
par(mfrow=c(1,2))

for (dataset in c("mtu173", "mtu_broad")) {

	inputdir = paste(prefix,"datasets/",dataset,"/ex",mut_type,"/comparison/stats_", subset, "/", sep="")

	print(inputdir)
	score_files = list.files(path = inputdir)

	print(score_files)

	score_areas = c()
	score_colors = c()

	input_fn = paste(inputdir, "/ws/0.txt", sep="")
	print(input_fn)

	data = read.table(input_fn, header=FALSE, sep="\t")

	print("read")

	if (dataset == "mtu_broad") {
		if (subset == "sel3") {
			positives = 75
			negatives = 870
		} else {
			positives = 220
			negatives = 725
		}
	} else if (dataset == "mtu173") {
		if (subset == "sel3") {
			positives = 39
			negatives = 1450
		} else {
			positives = 113
			negatives = 1376	
		}
	} else if (ncol(data) > 4) {
		positives = data[1,1] + data[1,4]
		negatives = data[1,2] + data[1,3]
	} else {
		positives = 0
		negatives = 0
	}

	scores = c("mi", "or", "r-tgh", "ws", "lh", "ts")
	#scores = c("mi_x0", "or_x0", "r-fcctw2_x0", "ws_x0", "lh_x0")

	if (dataset == "mtu173"){ 
		main_txt = paste("PR curves for the mtu173 dataset: ", toString(positives), " positives; " , toString(negatives), " negatives", sep="")
	} else {
		main_txt = paste("PR curves for the mtu_broad dataset: ", toString(positives), " positives; " , toString(negatives), " negatives", sep="")
	}
	plot(c(), cex.main=1.5, cex.lab = 1.3, xlab="tpr (recall) = TP/(TP+FN)", ylab="precision = TP/(TP+FP)", main=main_txt, xlim=c(0.0,1.0), ylim=c(0.0, 1.0))

	for (score_file in score_files) {
		score = strsplit(score_file, "[.]")[[1]][1]
		print(score)

		if (score %in% scores){

			input_fn = paste(inputdir, "/",score,".txt", sep="")
			print(input_fn)
			data = read.table(input_fn, header=FALSE, sep="\t")

			prec = c(data[1,2], data[,2])
			recall = c(0.0, data[,3])
			n = length(prec)

			area = 0.0

			for (i in 2:n) {
				area = area + 0.5*(prec[i] + prec[i-1])*(recall[i]-recall[i-1])
			}

			color = rgb(runif(1),runif(1),runif(1)) 

			score_full = scoreFullName(score)
			score_desc = paste(score_full, " [AUC=" , round(area,2) , "]", sep="")
			score_areas[[score_desc]] = area
			score_colors[[score_desc]] = color

			points(prec~recall, col=color, pch=18, lwd=1.5, type="l", xlim=c(0.0,1.0), ylim=c(0.0, 1.0))
		}
	}

	score_desc_sorted = names(sort(score_areas, decreasing = T))
	score_colors_sorted = c()

	for (score_desc in score_desc_sorted) {
		score_colors_sorted = c(score_colors_sorted, score_colors[[score_desc]])		
	}

	legend("topright", fill=score_colors_sorted, legend = score_desc_sorted, ncol=1, cex=1.2)

}
dev.off()

